

// Copyright: Hans de Nivelle, June 2006.
// Modified for Bliksem, September 2006. 


// This file is part of Geo2007F.
//
// Geo 2007F is free software: You can distribute it and/or modify
// it under the terms of the GNU General Public License, Version 3,
// as published by the Free Software Foundation.
//
// Geo2007F is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// Version 3 along with Geo2007F. It not, see 
// < http://www.gnu.org/licenses/ >



#include "expression.h"


namespace prolog 
{

   expression::expression( const expression& other )
      : repr( other. repr ) 
   {
      ASSERT( repr -> reference_count ); 
      ++ ( repr -> reference_count );
   }


   void expression::operator = ( expression other )
   {
      -- ( repr -> reference_count );
      if( ( repr -> reference_count ) == 0 )
         delete repr;

      repr = other. repr;
      ASSERT( repr -> reference_count );
      ++ ( repr -> reference_count );
   }


   expression::~expression( ) 
   {
      -- ( repr -> reference_count );
      if( ( repr -> reference_count ) == 0 )
         delete repr;
   }




   expression::expression( funcpred f, 
		           const std::list< expression > & subexpressions )
      : repr( new expression_node( f, subexpressions ))
   {
      ASSERT( f. getarity( ) == subexpressions. size( )); 
      ( repr -> reference_count ) = 1;
   }

   expression::expression( funcpred f, 
		           const std::vector< expression > & subexpressions )
      : repr( new expression_node( f, subexpressions ))
   {
      ASSERT( f. getarity( ) == subexpressions. size( ));
      ( repr -> reference_count ) = 1;
   }


   std::list< expression > :: const_iterator 
   expression::subexpressions_begin( ) const
   {
      return ( repr -> subexpressions. begin( ));
   }

   std::list< expression > :: const_iterator 
   expression::subexpressions_end( ) const
   {
      return ( repr -> subexpressions. end( ));
   }


   funcpred expression::getconstructor( ) const
   {
      return ( repr -> func );
   }

   unsigned int expression::getnrsubexpressions( ) const
   {
      return ( repr -> subexpressions. size( ));
   }



   std::ostream& operator << ( std::ostream& stream, 
		               const expression& exp )
   {
      stream << exp. getconstructor( );
      std::list< expression > :: const_iterator 
	     p1 = exp. subexpressions_begin( );
      std::list< expression > :: const_iterator 
	     p2 = exp. subexpressions_end( );

      if( p1 != p2 )
      {
         stream << "( "; 
         for( std::list< expression > :: const_iterator p = p1; p != p2; ++ p )
         {
            if( p != p1 )
               stream << ", ";
	    stream << *p;
         }
	 stream << " )"; 
      }
      return stream;
   }

} 



