
// Copyright: Hans de Nivelle.


// This file is part of Geo2007F.
//
// Geo 2007F is free software: You can distribute it and/or modify
// it under the terms of the GNU General Public License, Version 3,
// as published by the Free Software Foundation.
//
// Geo2007F is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// Version 3 along with Geo2007F. It not, see 
// < http://www.gnu.org/licenses/ >



#include "statisticscollector.h"



void statisticscollector::clear( ) 
{
   sum = 0.0;
   nr = 0;
   distr. clear( );
}


void statisticscollector::insert( unsigned int x )
{
   sum += static_cast< double > ( x );

   unsigned int ind = getindex( x );
   while( ind >= distr. size( ))
      distr. push_back(0);

   ++ distr [ ind ];

   ++ nr; 
}


unsigned int statisticscollector::getindex( unsigned x )
{
   if( x == 0 )
      return 0;

   unsigned int step1 = 1;
   unsigned int index = 1;

   while( step1 * 10 <= x )
   {
      step1 = step1 * 10;
      index += 9;
   }

   unsigned int step2 = step1;
   while( step1 + step2 <= x )
   {
      step1 += step2;
      ++ index; 
   }
   return index; 
}


unsigned int statisticscollector::getlowest( unsigned int i )
{
   if( i == 0 )
      return 0;

   -- i;

   unsigned int i0 = i / 9; 
   unsigned int i1 = i % 9;

   unsigned int low = 1;
   while( i0 )
   {
      -- i0;
      low *= 10;
   }

   low *= ( i1 + 1 );
   return low;
}




std::ostream& operator << ( std::ostream& stream,
		            const statisticscollector& stat )
{
   stream << "Statistics:\n";
   if( stat. nr != 0 )
   {
      stream << "nr =          " << stat. nr << "\n";
      stream << "average =     " << ( stat. sum ) / stat. nr;
      stream << "\n\n";

      stream << "Distribution:\n"; 
      for( unsigned int i = 0; i < stat. distr. size( ); ++ i )
      {
         stream << "   ";
	 stream << statisticscollector::getlowest(i) << "   :   ";
	 stream << stat. distr[i] << "\n";
      }
      stream << "\n";
   }

   return stream;
}



