
// Copyright: Hans de Nivelle


// This file is part of Geo2007F.
//
// Geo 2007F is free software: You can distribute it and/or modify
// it under the terms of the GNU General Public License, Version 3,
// as published by the Free Software Foundation.
//
// Geo2007F is distributed in the hope that it will be useful, but 
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License 
// Version 3 along with Geo2007F. It not, see 
// < http://www.gnu.org/licenses/ >


#include "unsignedsubst.h"

namespace swll
{

   unsignedsubst::unsignedsubst( variable v )
      : repr( std::vector< assignment > ( v. getunsigned( ) + 1 ))  
   {
   } 


   void unsignedsubst::clear( ) 
   {
      repr. clear( );
      assigned. clear( );
   }


   variable unsignedsubst::sup_variables( ) const
   {
      variable res;

      for( std::vector< variable > :: const_iterator 
              p = assigned. begin( );
	      p != assigned. end( );
	      ++ p )
      {
         variable v = *p;
	 ++ v;
	 if( v > res )
            res = v;
      }
      return res;
   }


   unsigned int unsignedsubst::sup_unsigneds( ) const
   {
      unsigned int res = 0;
      for( std::vector< variable > :: const_iterator
              p = assigned. begin( );
	      p != assigned. end( ); 
	      ++ p )
      {
         unsigned int k = repr [ p -> getunsigned( ) ]. val + 1;
	 if( k > res )
            res = k;
      }
      return res;
   }


   std::ostream& operator << ( std::ostream& stream,
		               const unsignedsubst& s )
   {
      stream << "unsignedsubst( ";
      for( unsigned int i = 0; i < s. size( ); ++ i )
      {
         if( i != 0 )
            stream << ", ";
	 stream << s. nthvar(i) << " := " << * s. lookup( s. nthvar(i) );
      }
      stream << " )";
      return stream;
   }

   
} 


